summaryrefslogtreecommitdiff
path: root/src
diff options
context:
space:
mode:
authorKay Sievers <kay@vrfy.org>2013-01-04 19:08:08 +0100
committerKay Sievers <kay@vrfy.org>2013-01-04 19:08:08 +0100
commit1328f66ad16b5afeb5684858c27e121a46c1959e (patch)
treefc76ff4b7b279f4842e4184eee237ce5985c4cf1 /src
parent48a849ee17fb25e0001bfcc0f28a4aa633d016a1 (diff)
udev: net_id - append "dev_id" value if needed
Diffstat (limited to 'src')
-rw-r--r--src/libudev/libudev-private.h1
-rw-r--r--src/libudev/libudev-util.c23
-rw-r--r--src/udev/udev-builtin-net_id.c58
3 files changed, 59 insertions, 23 deletions
diff --git a/src/libudev/libudev-private.h b/src/libudev/libudev-private.h
index 5b7f0021e..1b8638470 100644
--- a/src/libudev/libudev-private.h
+++ b/src/libudev/libudev-private.h
@@ -165,6 +165,7 @@ int util_log_priority(const char *priority);
size_t util_path_encode(const char *src, char *dest, size_t size);
void util_remove_trailing_chars(char *path, char c);
size_t util_strpcpy(char **dest, size_t size, const char *src);
+size_t util_strpcpyf(char **dest, size_t size, const char *src, ...) __attribute__((format(printf, 3, 4)));
size_t util_strpcpyl(char **dest, size_t size, const char *src, ...) __attribute__((sentinel));
size_t util_strscpy(char *dest, size_t size, const char *src);
size_t util_strscpyl(char *dest, size_t size, const char *src, ...) __attribute__((sentinel));
diff --git a/src/libudev/libudev-util.c b/src/libudev/libudev-util.c
index cb9ed9c84..b55bf75bf 100644
--- a/src/libudev/libudev-util.c
+++ b/src/libudev/libudev-util.c
@@ -321,14 +321,32 @@ size_t util_strpcpy(char **dest, size_t size, const char *src)
if (size > 1)
*dest = mempcpy(*dest, src, size-1);
size = 0;
- *dest[0] = '\0';
} else {
if (len > 0) {
*dest = mempcpy(*dest, src, len);
size -= len;
}
- *dest[0] = '\0';
}
+ *dest[0] = '\0';
+ return size;
+}
+
+size_t util_strpcpyf(char **dest, size_t size, const char *src, ...)
+{
+ va_list va;
+ int i;
+
+ va_start(va, src);
+ i = vsnprintf(*dest, size, src, va);
+ if (i < (int)size) {
+ *dest += i;
+ size -= i;
+ } else {
+ *dest += size;
+ size = 0;
+ }
+ va_end(va);
+ *dest[0] = '\0';
return size;
}
@@ -343,7 +361,6 @@ size_t util_strpcpyl(char **dest, size_t size, const char *src, ...)
src = va_arg(va, char *);
} while (src != NULL);
va_end(va);
-
return size;
}
diff --git a/src/udev/udev-builtin-net_id.c b/src/udev/udev-builtin-net_id.c
index d8d450817..d5db762e8 100644
--- a/src/udev/udev-builtin-net_id.c
+++ b/src/udev/udev-builtin-net_id.c
@@ -30,12 +30,12 @@
* ww -- wwan
*
* Type of names:
* o<index> -- on-board device index number
* s<slot>[f<function>] -- hotplug slot index number
* x<MAC> -- MAC address
* p<bus>s<slot>[f<function>] -- PCI geographical location
* p<bus>s<slot>[f<function>][u<port>][...][c<config>][i<interface>]
* -- USB port number chain
*
* All multi-function PCI devices will carry the [f<function>] number in the
* device name, including the function 0 device.
@@ -134,11 +134,11 @@ static int dev_pci_onboard(struct udev_device *dev, struct netnames *names) {
}
/* read the 256 bytes PCI configuration space to check the multi-function bit */
-static bool is_pci_singlefunction(struct udev_device *dev) {
+static bool is_pci_multifunction(struct udev_device *dev) {
char filename[256];
FILE *f;
char config[64];
- bool single = false;
+ bool multi = false;
snprintf(filename, sizeof(filename), "%s/config", udev_device_get_syspath(dev));
f = fopen(filename, "re");
@@ -148,11 +148,11 @@ static bool is_pci_singlefunction(struct udev_device *dev) {
goto out;
/* bit 0-6 header type, bit 7 multi/single function device */
- if ((config[PCI_HEADER_TYPE] & 0x80) == 0)
- single = true;
+ if ((config[PCI_HEADER_TYPE] & 0x80) != 0)
+ multi = true;
out:
fclose(f);
- return single;
+ return multi;
}
static int dev_pci_slot(struct udev_device *dev, struct netnames *names) {
@@ -160,6 +160,10 @@ static int dev_pci_slot(struct udev_device *dev, struct netnames *names) {
unsigned int bus;
unsigned int slot;
unsigned int func;
+ unsigned int dev_id = 0;
+ size_t l;
+ char *s;
+ const char *attr;
struct udev_device *pci = NULL;
char slots[256];
DIR *dir;
@@ -168,13 +172,23 @@ static int dev_pci_slot(struct udev_device *dev, struct netnames *names) {
int hotplug_slot = 0;
int err = 0;
- /* compose a name based on the raw kernel's PCI bus, slot numbers */
if (sscanf(udev_device_get_sysname(names->pcidev), "0000:%x:%x.%d", &bus, &slot, &func) != 3)
return -ENOENT;
- if (func == 0 && is_pci_singlefunction(names->pcidev))
- snprintf(names->pci_path, sizeof(names->pci_path), "p%ds%d", bus, slot);
- else
- snprintf(names->pci_path, sizeof(names->pci_path), "p%ds%df%d", bus, slot, func);
+
+ /* kernel provided multi-device index */
+ attr = udev_device_get_sysattr_value(dev, "dev_id");
+ if (attr)
+ dev_id = strtol(attr, NULL, 16);
+
+ /* compose a name based on the raw kernel's PCI bus, slot numbers */
+ s = names->pci_path;
+ l = util_strpcpyf(&s, sizeof(names->pci_path), "p%ds%d", bus, slot);
+ if (func > 0 || is_pci_multifunction(names->pcidev))
+ l = util_strpcpyf(&s, l, "f%d", func);
+ if (dev_id > 0)
+ l = util_strpcpyf(&s, l, "d%d", dev_id);
+ if (l == 0)
+ names->pci_path[0] = '\0';
/* ACPI _SUN -- slot user number */
pci = udev_device_new_from_subsystem_sysname(udev, "subsystem", "pci");
@@ -215,10 +229,14 @@ static int dev_pci_slot(struct udev_device *dev, struct netnames *names) {
closedir(dir);
if (hotplug_slot > 0) {
- if (func == 0 && is_pci_singlefunction(names->pcidev))
- snprintf(names->pci_slot, sizeof(names->pci_slot), "s%d", hotplug_slot);
- else
- snprintf(names->pci_slot, sizeof(names->pci_slot), "s%df%d", hotplug_slot, func);
+ s = names->pci_slot;
+ l = util_strpcpyf(&s, sizeof(names->pci_slot), "s%d", hotplug_slot);
+ if (func > 0 || is_pci_multifunction(names->pcidev))
+ l = util_strpcpyf(&s, l, "f%d", func);
+ if (dev_id > 0)
+ l = util_strpcpyf(&s, l, "d%d", dev_id);
+ if (l == 0)
+ names->pci_path[0] = '\0';
}
out:
udev_device_unref(pci);
@@ -30,12 +30,12 @@
*
* Type of names:
- * o<index> -- on-board device index number
- * s<slot>[f<function>] -- hotplug slot index number
- * x<MAC> -- MAC address
- * p<bus>s<slot>[f<function>] -- PCI geographical location
- * p<bus>s<slot>[f<function>][u<port>][...][c<config>][i<interface>]
- * -- USB port number chain
+ * o<index> -- on-board device index number
+ * s<slot>[f<function>][d<dev_id>] -- hotplug slot index number
+ * x<MAC> -- MAC address
+ * p<bus>s<slot>[f<function>][d<dev_id>] -- PCI geographical location
+ * p<bus>s<slot>[f<function>][u<port>][..][c<config>][i<interface>]
+ * -- USB port number chain
*
* All multi-function PCI devices will carry the [f<function>] number in the